﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Navigation;
using Microsoft.VisualStudio.ExtensibilityHosting;
using Microsoft.LightSwitch.Sdk.Proxy;
using Microsoft.LightSwitch.Runtime.Shell;
using Microsoft.LightSwitch.Runtime.Shell.View;
using Microsoft.LightSwitch.Runtime.Shell.ViewModels.Navigation;
using Microsoft.LightSwitch.Runtime.Shell.ViewModels.Notifications;
using Microsoft.LightSwitch.BaseServices.Notifications;
using Microsoft.LightSwitch.Client;
using Microsoft.LightSwitch.Framework.Client;
using Microsoft.LightSwitch.Details;
using Microsoft.LightSwitch.Threading;

namespace SingleScreen
{
		/// <summary>
		/// Represents the main application control.
		/// </summary>
		public partial class SingleScreenShellPage : Page, IWeakNotificationSupport
		{
				private IServiceProxy serviceProxy;
				private List<object> weakHelperObjects = new List<object>();

				public SingleScreenShellPage()
				{
						this.InitializeComponent();

						this.Loaded += new RoutedEventHandler(SingleScreenShellPage_Loaded);

						//// Subscribe to the screen selection changed notification in order to get
						//// notified when the active screen referred to by the active screen view
						//// model changes.
						this.ServiceProxy.NotificationService.Subscribe(typeof(ScreenSelectionChangedNotification), this.OnCurrentScreenChanged);
						this.ServiceProxy.NotificationService.Subscribe(typeof(ScreenOpenedNotification), this.OnScreenOpened);
						this.ServiceProxy.NotificationService.Subscribe(typeof(ScreenClosedNotification), this.OnScreenClosed);
						this.serviceProxy.NavigationViewModel.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(NavigationViewModel_PropertyChanged);
						this.LoadViewModels();
				}

				void NavigationViewModel_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
				{
					if (e.PropertyName=="NavigationItems")
					{
						INavigationScreen link = FindLink();
						IBusinessCommand businessCommand = (IBusinessCommand)link.ExecutableObject;
						IDispatcherObject dispatcher = (IDispatcherObject)businessCommand;

						dispatcher.Dispatcher.BeginInvoke(delegate()
						{
							if (businessCommand.CanExecute)
								businessCommand.Execute();
						});
					}
				}

				private IServiceProxy ServiceProxy
				{
						get
						{
								// Get the service proxy which provides access to the needed
								// LightSwitch services.
								if (null == this.serviceProxy)
										this.serviceProxy = VsExportProviderService.GetExportedValue<IServiceProxy>();

								return this.serviceProxy;
						}
				}

				#region IWeakNotificationSupport methods
				public void HoldObject(object o)
				{
						this.weakHelperObjects.Add(o);
				}
				#endregion

				private void LoadViewModels()
				{
					
					//// Get the view models loaded (and cached) by the proxy service.
						////
						//// A GUI shell will probably end up binding to these view models.  Since this
						//// shell doesn't do that, the view models need to be loaded manually.
					if (this.ServiceProxy.NavigationViewModel == null)
					{
						throw new System.InvalidOperationException("Unable to retrieve navigation view model");
					}
					else
					{
						// The navigation view model will only return those links that can be executed.  In order
						// to do that, it will call the CanExecute() method on the links - which is an
						// asynchronous operation.  So, since this shell doesn't bind to the view models I need
						// to "prime" the view model by asking for the links.  If I don't do this, the first
						// command entered in the shell will not work (all commands after that will work).
						this.ServiceProxy.NavigationViewModel.NavigationItems.Count();
					}

					if (this.ServiceProxy.CommandsViewModel == null)
						throw new System.InvalidOperationException("Unable to retrieve commands view model");

					if (this.ServiceProxy.ActiveScreensViewModel == null)
					{
						throw new System.InvalidOperationException("Unable to retrieve active screens view model");
					}

				}
				private void OnScreenOpened(Notification n)
				{
						ScreenOpenedNotification screenOpenedNotification = (ScreenOpenedNotification)n;
						IScreenObject screenObject = screenOpenedNotification.Screen;
						IScreenView view = this.ServiceProxy.ScreenViewService.GetScreenView(screenObject);

						// Set the screen area to the UI element of the screen view.
						this.ScreenArea.Content = view.RootUI;

						// Set the currently active screen in the active screens view model.
						this.ServiceProxy.ActiveScreensViewModel.Current = screenObject;
				}
				private void OnScreenClosed(Notification n)
				{
					// A screen has been close and therefore removed from the application's
					// collection of active screens.  Get the IScreenView from the active screen
					// object and set the screen area content to its RootUI property.
					//
					int count = ClientApplicationProvider.Current.ActiveScreens.Count();
					if (count > 0)
					{
						ScreenClosedNotification screenClosedNotification = (ScreenClosedNotification)n;
						IScreenObject screenObject = screenClosedNotification.Screen;
						IScreenView view = this.ServiceProxy.ScreenViewService.GetScreenView(screenObject);
						this.ScreenArea.Content = view.RootUI;
					}
					else
						this.ScreenArea.Content = null;
				}
				private void OnCurrentScreenChanged(Notification n)
				{
					// The current active screen has changed.  Get the screen object from
					// the notification arguments and update the screen area content
					// accordingly.
					ScreenSelectionChangedNotification sscn = (ScreenSelectionChangedNotification)n;
					IScreenObject screenObject = sscn.SelectedScreen;
					IScreenView view = this.ServiceProxy.ScreenViewService.GetScreenView(screenObject);

					this.ScreenArea.Content = view.RootUI;
				}
				private void SingleScreenShellPage_Loaded(object sender, RoutedEventArgs e)
				{
						try
						{
								System.Windows.Browser.HtmlPage.Plugin.Focus();
						}
						catch (InvalidOperationException)
						{
						}
				}
				private INavigationScreen FindLink()
				{
					// Recursively descend into the navigation object model to find the
					// navigation item with the specified name.
					INavigationScreen link = null;

					foreach (INavigationItem navItem in this.ServiceProxy.NavigationViewModel.NavigationItems)
					{
						if (navItem is INavigationGroup)
							link = this.FindLink((INavigationGroup)navItem);
						else if (navItem is INavigationScreen)
						{
								link = (INavigationScreen)navItem;
						}

						if (null != link)
							break;
					}

					return link;
				}
				private INavigationScreen FindLink(INavigationGroup navGroup)
				{
					INavigationScreen link = null;

					foreach (INavigationItem navItem in navGroup.Children)
					{
						if (navItem is INavigationGroup)
							link = this.FindLink((INavigationGroup)navItem);
						else if (navItem is INavigationScreen)
						{
							link = (INavigationScreen)navItem;
						}

						if (null != link)
							break;
					}

					return link;
				}
		}
}
