﻿//--------------------------------------------------------------------
// <copyright company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    Information Contained Herein Is Proprietary and Confidential.
// </copyright>
//--------------------------------------------------------------------

using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.Reflection;
using System.Resources;

using Microsoft.LightSwitch.Model;

namespace SingleScreen.Metadata
{
    [Export(typeof(IModuleDefinitionLoader))]
    [ModuleDefinitionLoader(SingleScreenModule.Name)]
    internal class SingleScreenModuleLoader : IModuleDefinitionLoader
    {
        #region IModuleDefinitionLoader Members

        IEnumerable<Stream> IModuleDefinitionLoader.LoadModelFragments()
        {
            Assembly assem = Assembly.GetExecutingAssembly();
            string[] resources = assem.GetManifestResourceNames();
            foreach (string resource in resources)
            {
                foreach (string fragment in ModelFragments)
                {
                    if (resource.Contains(fragment))
                    {
                        Stream fragmentStream = assem.GetManifestResourceStream(resource);
                        yield return fragmentStream;

                    }
                }
            }
        }

        ResourceManager IModuleDefinitionLoader.GetModelResourceManager()
        {
            return SingleScreen.Resources.SingleScreen_Resources.ResourceManager;
        }

        #endregion

        #region Static Members

        private static string[] ModelFragments = {
            "SingleScreenModule.lsml",
            "SingleScreenPresentation.lsml"
        };

        private static string ResourceNamespace = typeof(SingleScreenModuleLoader).Namespace;

        #endregion

        #region Constants

        private const string ResourcePathFormat = "{0}.{1}";

        #endregion
    }
}
